        name      DRSA1
        title     Driver Rotore Satellitare Autarchico
;     Questo driver e' molto simile al ROT9.COM, scritto a sua volta a partire
;     dal ROTORDUM.ASM (ovvero il DummyKCT).
;     Invia sulla COMn (non impostabile, bisogna riassemblare...)
;     i valori relativi a azimuth e elevazione con un semplice formato:
;     <azim flag> <azim val> <elev flag> <elev val>
;     con:  <azim flag> = e0h  e  <elev flag> = eeh
;     I due valori sono divisi per due, cos  possibile trasmetterli
;     in un solo byte: la risoluzione di 2 si ritiene per ora sufficiente.
;
;
;                 Prima versione messa a punto l'11/11/1995
;                 Modifiche:
;                       24/02/1996 prova con il vero rotore (solo az)
;                       10/5/96 messo a punto il ROT9.ASM
;                       23/2/97 iniziata la riscrittura del driver definitivo


;-----------------------------------------------------------------------------
; Inizializzazioni necessarie per il file .COM (e TSR)
;-----------------------------------------------------------------------------

_TEXT	segment word public 'CODE'

        ORG     100h                    ; .COM

	assume	cs:_TEXT, ds:_TEXT, es:_TEXT, ss:_TEXT

;-----------------------------------------------------------------------------
; Lanciando questo .COM si va alla routine di installazione, poi una parte del
; programma, cioe' quella da <isr:> a <endisr:> viene lasciata in memoria e
; costituisce le routine di servizio dell'interrupt 63 (isr).
;-----------------------------------------------------------------------------
start:	jmp	install

enabled db      1       ; Abilitazione: non ne ho capito l'utilita' ma si lascia
                        ; per compatibilita' con il dummyKCT, en=1, dis=0
azim    dw      0       ; Valore attuale di azimuth
elev    dw      0       ; Valore attuale di elevazione
oldazim dw      0       ; Valori della chiamata precedente per
oldelev dw      0       ; trasmettere solo se c'e' stata una variazione
flip    db      0       ; Arrovesciamento antenne per spostare il punto morto a
                        ; sud anziche a nord in modo da non dover fare tutto il
                        ; giro a meta' di certi passaggi

COM   equ   3                 ; COM1 = 0, COM2 = 1, COM3 = 2, COM4 =3


;-----------------------------------------------------------------------------
; Interrupt Service Routine
;-----------------------------------------------------------------------------

	assume	ds:nothing, es:nothing, ss:nothing

isr:        jmp     servsel

            org     isr + 10
            db      "RotorDRV"      ; Firma, affinche il programma chiamante
                                    ; riconosca che il driver e' presente (vedi
                                    ; l' MRC.C. Questa firma si deve trovare 10
                                    ; (decimale) byte dopo l'inizio della isr

servsel     proc   near
                                    ; Scelta della routine per svolgere la
                                    ; funzione richiesta in base a AH
            cmp   ah,0
            je    readstat
            cmp   ah,3
            je    readpos
            cmp   ah,4
            je    setpos
            cmp   ah,9
            je    enable
            cmp   ah,40h
            je    flipctrl
            cmp   ah,41h
            je    flipread
            iret
servsel     endp

;-----------------------------------------------------------------------------
; Funzioni chiamate dall'interrupt
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
; Readstat: restituisce lo stato del driver, ovvero del rotore, ma per ora non
;           e' realmente funzionante, anche perche' non si conosce il
;           significato dei codici di ritorno. L'unica cosa che ho
;           scoperto e' che l'Instant Track dice che il rotore e' disabilitato
;           se questa funzione ritorna ffh, altrimenti funziona; per
;           compatibilita' con il DummyKCT fi fa ritornare 00h.
;
;     Ingresso:
;           AH=0
;     Uscita:
;           AL=0 (vedi sopra)
;-----------------------------------------------------------------------------

readstat    proc  near
            mov   al,0h
            iret
readstat    endp


;-----------------------------------------------------------------------------
; Readpos:  restituisce l'ultima posizione trasmessa al rotore (in seguto
;           dovra' essere l'ultima posizione che il rotore ha ritrasmesso
;           indietro, ma per ora il protocollo non e' cosi' raffinato)
;
;     Ingresso:
;           AH=3
;     Uscita:
;           DX=<azimuth>
;           CX=<elevazione>
;-----------------------------------------------------------------------------

;readpos     proc  near
;            mov   ax,cs:oldelev
;            sal   ax,1        ; oldelev e' diviso per due, va rimoltiplicato
;            mov   cx,ax
;            mov   bx,cs:oldazim
;            sal   bx,1        ; oldazim e' diviso per due, va rimoltiplicato
;            mov   dx,bx
;            iret
;readpos     endp

; !!! nel modo sopra ci sono dei problemi, anche se e' quello corretto !!!
; Questo invece funziona ma non mi piace.

readpos     proc  near
            mov   ax,cs:elev
            mov   cx,ax
            mov   bx,cs:azim
            mov   dx,bx
            iret
readpos     endp

; Mi sarebbe piaciuto mettere le routine in ordine di AH, ma la Setpos e' cosi'
; lunga che il salto near non ce la fa!

;-----------------------------------------------------------------------------
; Enable:   imposta il flag di abilitazione o meno di questo driver, vedi le
;           note della funzione Readstat.
;     Ingresso:
;           AH=9
;           AL=<enable>  1=enable, 0=disable
;     Uscita:
;           ---
;-----------------------------------------------------------------------------

enable      proc    near
            mov     enabled,al
            iret
enable      endp


;-----------------------------------------------------------------------------
; Flipctrl: imposta o meno la modalita flip. Questa modalita' non e' ancora
;           chiara, soprattutto in relazione all'uso che ne deve fare la
;           Control-Box, percio non e' da considerarsi stabile.
;           Il valore AH=40 per questa funzione l'ho preso senza sapere se era
;           libero o meno, ma non sembra ci siano conflitti con l'Instant Track
;           o l' Orbitdrv.
;     Ingresso:
;           AH=40
;           AL=<flip>  1=flip, 0=no flip
;     Uscita:
;           ---
;-----------------------------------------------------------------------------

flipctrl    proc  near
            mov   flip,al
            iret
flipctrl    endp


;-----------------------------------------------------------------------------
; Flipread: restituisce lo stato della modalita' flip, vedi descrizione funzione
;           Flipctrl.
;           Il valore AH=40 per questa funzione l'ho preso senza sapere se era
;           libero o meno, ma non sembra ci siano conflitti con l'Instant Track
;           o l' Orbitdrv.
;
;     Ingresso:
;           AH=41
;     Uscita:
;           AL=<flip>  1=flip, 0=no flip
;-----------------------------------------------------------------------------

flipread    proc  near
            mov al,flip
            iret
flipread    endp


;-----------------------------------------------------------------------------
; Setpos:   trasmette la posizione al rotore solo se questa e' cambiata
;           rispetto all'interrupt precedente. Il DummyKCT dice che i valori
;           validi sono "(0-359, binary)" per l'azimuth, e
;           "(-90 to 269, signed binary)" per l'elevazione, inoltre c'e' la
;           "Note: If either value is invalid the entire command will be
;           ignored.".
;           In effetti questo driver usa solo valori 0-359 per l'azimuth e 0-180
;           per l'elevazione (0-90 o 90-180 se flip) e se l'elevazione e'
;           negativa non trasmette nulla.
;
;     Ingresso:
;           AH=4
;           DX=<azimuth>
;           CX=<elevazione>
;     Uscita:
;           ---
;-----------------------------------------------------------------------------

setpos      proc  near
            cmp   enabled,0       ; se il driver e' disabilitato esci
            je    sp_end

            mov   cs:azim,dx
            mov   cs:elev,cx
            test  cx,8000h      ;se elev<0 o elev>180 non trasmettere nulla
            jnz   sp_end
            cmp   cx,00b4h
            jg    sp_end

            sar   dx,1              ; sia azim che elev vengono divisi per
            sar   cx,1              ; due cosi' basta un byte per trasmetterli
                                    ; (risoluzione di 2 gradi)

            cmp   flip,0
            jz    save
            mov   bx,00b4h
            sub   bx,dx
            mov   dx,bx
            mov   bx,005ah
            sub   bx,cx
            mov   cx,bx

save:       cmp   cs:oldazim,dx     ; se l'az non e' cambiato non trasmettere
            je    testel


sendaz:
            push      dx
            mov       cs:oldazim,dx
            mov       ax,01e0h
            mov       dx,COM
            int       14h               ; flag azim
            pop       dx
            mov       al,dl
            mov       ah,1
            mov       dx,COM
            int       14h               ; valore azim

testel:     cmp       cs:oldelev,cx
            je        sp_end

        push      dx                ; ritardo per dare tempo al PIC
        push      cx                ; di ricevere l'eventuale dato
        mov       ah,86h            ; dell'elevazione prima di dover
        mov       cx,0000h          ; gestire un'altro suo interrupt
        mov       dx,0258h          ; (porta seriale in ricezione)
        int       15h               ;
        pop       cx                ;
        pop       dx                ;

sendel:
            push      cx
            mov       cs:oldelev,cx
            mov       ax,01eeh
            mov       dx,COM
            int       14h               ; flag elev
            pop       cx
            mov       al,cl
            mov       ah,1
            mov       dx,COM
            int       14h               ; valore elev

sp_end:     iret

setpos      endp


; La funzione del DOS che fa divenatare questo codice un TSR deve sapere quale
; porzione di codice debba essere lascaita in memoria, e l'etichetta endisr
; serve per dire dov'e' la fine; l'inizio e l'etichetta isr.

endisr      db    00h


;-----------------------------------------------------------------------------
; Questa funzione, a cui il programma salta immediatamente dopo il caricamento
; in memoria, configura la porta seriale e installa le routine di servizio per
; l'interrupt 63.
;-----------------------------------------------------------------------------
install     proc  near

;-----------------------------------------------------------------------------
; Configurazione porta seriale
;-----------------------------------------------------------------------------
      mov       ax,00e3h      ; 9600 n 8 1
      mov       dx,COM
      int       14h           ; setup porta seriale

;-----------------------------------------------------------------------------
; Installazione isr
;-----------------------------------------------------------------------------

            mov   dx,offset isr     ; questo e' l'indirizo d'inizio delle isr
            mov   ax,2563h          ; l'interrupt e' il 63h
            int   21h               ; abilitazione dell'interrupt

            mov   dx,offset endisr  ; questo e' l'indirizzo finale delle isr
            add   dx,15             ; arrotondamento
            mov   cl,4
            shr   dx,cl             ; conversione nel numero di paragrafi da
                                    ; riservare per le routine isr che
                                    ; diventeranno TSR
            mov   ax,3100h
            int   21H               ; funzione DOS che fa diventare TSR (e che
                                    ; ovviamente non esce...)

install     endp

_TEXT       ends


            end   start
